using System;
using System.Collections;

namespace Printy.Library
{
	/// <summary>
	/// Erweiterte Konfiguration fr den HyperTextBuilder.
	/// </summary>
	public class HyperConfig
	{
    /// <summary>
    /// Hlt fest, ob Nonterminalsymbole mit Produktionen verlinkt werden sollen.
    /// </summary>
    public bool createLinks = true;

    /// <summary>
    /// CSS-Stylesheet
    /// Standardwert: null
    /// </summary>
    public string styleSheet = null;

    /// <summary>
    /// Dieser Bitvektor speichert jene Elemente, fr die eine Stilklasse erzeugt wird.
    /// Standardwert: []
    /// </summary>
    public BitSet styleSet = new BitSet();

    /// <summary>
    /// Abbildung von String auf CSS-Klassennamen.
    /// </summary>
    Hashtable styleMap;

    /// <summary>
    /// Schlssel zum Suchen
    /// </summary>
    StyleKey searchKey;

    /// <summary>
    /// Setzt einen Stil
    /// </summary>
    /// <param name="text">Betreffender Text</param>
    /// <param name="type">Typ des Textes</param>
    /// <param name="className">CSS-Klassenname</param>
    public void PutStyleClass(string text, int type, string className)
    {
      if (styleMap == null)
      {
        styleMap = new Hashtable();
        searchKey = new StyleKey();
      }
      styleMap[new StyleKey().Set(text,type)] = className;
    }

    /// <summary>
    /// Gibt eine CSS-Klasse fr einen Text.
    /// </summary>
    /// <param name="text">Betreffender Text</param>
    /// <param name="type">Typ des Textes</param>
    /// <returns>Den mit PutStyleClass() gesetzten CSS-Klassenname oder null,
    /// falls es keine Entsprechung gibt.
    /// </returns>
    public string GetStyleClass(string text, int type)
    {
      return (styleMap != null) ? (string)styleMap[searchKey.Set(text,type)] : null;
    }
	}

  /// <summary>
  /// Ein Schlssel in der styleMap (Hashtable) des HyperTextBuilder.
  /// </summary>
  class StyleKey
  {
    /// <summary>
    /// Text
    /// </summary>
    string text;

    /// <summary>
    /// Typ des Textes. ALL ist auch erlaubt
    /// </summary>
    int type;

    /// <summary>
    /// Setzt die Werte des Schlssels und gibt diesen zurck.
    /// </summary>
    /// <param name="text">Text</param>
    /// <param name="type">Typ</param>
    /// <returns>this</returns>
    internal StyleKey Set(string text, int type)
    {
      this.text = text;
      this.type = type;
      return this;
    }

    /// <summary>
    /// Vergleicht diesen Schlssel mit einem anderen. Es werden Text und Typ verglichen; falls
    /// der Typ ALL ist, wird nur der Text verglichen.
    /// </summary>
    /// <param name="obj">Vergleichsobjekt</param>
    /// <returns>true, falls die StyleKey-Objekte gleich sind.</returns>
    public override bool Equals(object obj)
    {
      StyleKey key = (StyleKey)obj;
      return ((type == Config.ALL) || (key.type == Config.ALL) || (type == key.type)) && (text == key.text);
    }

    /// <summary>
    /// Gibt den HashCode des Textes zurck.
    /// </summary>
    /// <returns>text.GetHashCode()</returns>
    public override int GetHashCode()
    {
      return text.GetHashCode();
    }
  }
}
